﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;



public class pinSpawner : MonoBehaviour {

    //ピンの間隔(基準値)
    const float PIN_SPACE = 18.5f;
    const float PIN_WALL_SPACE = 6.95f; //ピンの両端と壁の間

    const float STANDARD_RATE = 1.0f / 20.0f;
    //判定開始から次の投球への開始が確定するまでの時間
    const float TURN_END_TIME = 4.0f;

    const int MAX_PIN_NUM = 1000;
    const int MIN_PIN_NUM = 10;

    [SerializeField]
    GameObject pinPrefab;
    [SerializeField]
    GameObject ballObj;
    [SerializeField]
    GameRuler Ruler;

    [SerializeField]
    GameObject gatterlane;

    UIController UIcontroll;

    //ピンの数を常にカウントし続ける
    [SerializeField]
    int nNumStandPin;
    int nOldStandPin;
    [SerializeField]
    int nNumKnocdownPin;

    [SerializeField]
    Vector3 PinSetPos;
    [SerializeField]
    int nNextPin;
    int nAllPinNum;

    [SerializeField]
    int nFramePinNum = 0;


    float fDetectStart;
    float fDetectEnd;
    [SerializeField]
    bool bTurnCheck;
    [SerializeField]
    bool bNextTurn;

    float fPosZ_LastColum;

    Vector3 PinWallMAX;
    Vector3 PinWallMIN;

	// Use this for initialization
	void Start () {
        nNumStandPin = 0;
        nOldStandPin = 0;
        nNumKnocdownPin = 0;
        fPosZ_LastColum = 0.0f;
        PinWallMAX = Vector3.zero;
        PinWallMIN = Vector3.zero;

        ResetAllFlag();

        SetPin();

        UIcontroll = GameObject.Find("UI").GetComponent<UIController>();
        //gatterlane = GameObject.Find("GatterLane");

	}

    // Update is called once per frame
    void Update()
    {
        if(Input.GetKey(KeyCode.S)){
            Strike();
        }

        //立っている本数を数える
        CountStandPin();

        //次の投球に移るかどうかの判定
        CheckPinMovement();

        //UIにセット
        UIcontroll.SetCombo(nNumKnocdownPin);
        UIcontroll.SetDrawStrike(nNumKnocdownPin == nAllPinNum);

	}

    void ResetAllFlag()
    {
        bTurnCheck = false;
        bNextTurn = false;
        fDetectStart = 0.0f;
        fDetectEnd = 0.0f;
    }

    public void RespawnPin() {
        //現在あるピンを全て消してからピンを再配置
        DeleatePin();
        SetPin();
        ResetAllFlag();
    }

    void SetPin()
    {
        nNextPin = Mathf.Max(Mathf.Min(nNextPin, MAX_PIN_NUM), MIN_PIN_NUM);

        float pin_space = PIN_SPACE * STANDARD_RATE;

        int nLine = 0;
        int nCntPin = 0;
        int nLinepin = 0;
        float z = 0.0f;
        for (nLine = 0; nCntPin < nNextPin; nLine++)
        {
            //三平方
            z = nLine * Mathf.Sqrt(3) * pin_space * 0.5f;
            for (nLinepin = 0; nLinepin <= nLine ; nLinepin++, nCntPin++)
            {
                if (nCntPin >= nNextPin) break;

                GameObject pinObj = (GameObject)Instantiate(pinPrefab);
                pinObj.transform.parent = transform;
                Vector3 pinPos = new Vector3(PinSetPos.x + nLine * (pin_space * 0.5f) - pin_space * nLinepin, PinSetPos.y + 1.0f, PinSetPos.z + z);
                pinPos += transform.position;   //spawnerの座標も加算する
                pinObj.transform.position = pinPos;

                pinObj.transform.localScale = new Vector3(3.0f, 3.0f, 3.0f);
            }
        }
        fPosZ_LastColum = PinSetPos.z + z;

        float fWallWidth = nLine * pin_space + (PIN_WALL_SPACE * STANDARD_RATE) * 2;
        //if (gatterlane.GetComponent<gatterSafe>())
        //{
            gatterlane.GetComponent<gatterSafe>().SetGatterSafePos(fWallWidth);
        //}
        PinWallMAX.x = PinSetPos.x + nLine * (pin_space * 0.5f) + 5.0f;
        PinWallMIN.x = PinSetPos.x + nLine * (pin_space * 0.5f) - pin_space * nLinepin -5.0f;
        PinWallMAX.y =  5.0f;
        PinWallMIN.y = -5.0f;
        PinWallMAX.z = PinSetPos.z + z + 5.0f;
        PinWallMIN.z = PinSetPos.z - 5.0f;

        PinWallMAX += transform.position;   //spawnerの座標も加算する
        PinWallMIN += transform.position;

        nAllPinNum = nNextPin;

        nNumStandPin = nAllPinNum;
        nNumKnocdownPin = 0;
        //SetPinID();
    }

    //void SetPinID() {
    //    foreach (var obj in GetComponentsInChildren<pinScript>()) {
    //        //各ピンにIDを割り振る
    //        int nID = obj.transform.GetSiblingIndex();
    //        obj.SetID(nID);
    //    }
    //}

    void DeleatePin() {
        foreach (var obj in GetComponentsInChildren<pinScript>()) {
            Destroy(obj.gameObject);
        }
    }

    void CountStandPin() {
        nOldStandPin = nNumStandPin;

        //カウントする前に位置次第でフラグを変更しておく
        KnockPin();

        //リセット
        nNumStandPin = 0;
        nNumKnocdownPin = 0;
        foreach (var obj in GetComponentsInChildren<pinScript>()) {
            //各ピンが立っているかチェック
            if (obj.GetStandFlag()) {
                nNumStandPin++;
            }
            else {
                nNumKnocdownPin++;

                //ピンが倒れたのを検出、投球の終了チェック開始フラグオン
                bTurnCheck = true;
            }
        }

        SetNowFramePinNum();
    }


    //1投してピンが残っている場合この関数を呼ぶ
    void RemovePin() {
        foreach (var obj in GetComponentsInChildren<pinScript>())
        {
            //各ピンが立っているかチェック
            if (!obj.GetStandFlag()) {
                //立っていなかったら殺す
                Destroy(obj.gameObject);
            }
            else {
                //立っているなら位置修正
                obj.FixedPosition();
            }
        }
        ResetAllFlag();
    }

    void CheckPinMovement() {

        if (bNextTurn) {
            //既に次の投球へ移行
            Ruler.NextTurn();
        }

        //チェックがオフの時だけ
        if (!bTurnCheck)
        {
            //ボールが一番後ろのピンが置かれているZ座標を超えたとき
            if (ballObj.transform.position.z > fPosZ_LastColum)// + this.transform.position.z)
            {
                bTurnCheck = true;

                //時間をそろえる
                fDetectStart = Time.realtimeSinceStartup;
                fDetectEnd = Time.realtimeSinceStartup;
            }
        }
        else {

            //倒れた数が変わらない
            if (nNumStandPin == nOldStandPin)
            {
                //時間を更新していく
                fDetectEnd = Time.realtimeSinceStartup;
            }
            else {
                //時間をそろえる
                fDetectStart = Time.realtimeSinceStartup;
                fDetectEnd = Time.realtimeSinceStartup;
            }

            if (fDetectEnd - fDetectStart >= TURN_END_TIME)
            {
                bNextTurn = true;
            }
        }
    }

    public int GetCollapsedPins()
    {
        return nNumKnocdownPin;
    }

    public int GetStandPins()
    {
        return nNumStandPin;
    }

    public int NowGamePinNum() {
        return nAllPinNum;
    }

    public void RelocatePin()
    {
        if (nNumStandPin == 0)//ターン数を拾ってくる
        {//全消去全配置
            RespawnPin();
        }
        else
        {//倒れてるの除去
            RemovePin();
        }
    }



    void Strike() {
        foreach (var obj in GetComponentsInChildren<pinScript>())
        {
            obj.transform.rotation = Quaternion.Euler(90.0f,0.0f,0.0f);
        }
    }

    void KnockPin() {
        foreach (var obj in GetComponentsInChildren<pinScript>()) {
            if (obj.GetStandFlag())
            {
                if (CheckOverLine(obj.transform.position))
                {
                    obj.SetStandPin(false);
                    obj.GetComponent<Rigidbody>().angularVelocity = new Vector3(3.0f, 0.0f, 0.0f);    //一応無理やり回転させる
                }
            }
            else
            {
                obj.GetComponent<Rigidbody>().angularVelocity = new Vector3(1.0f, 0.0f, 0.0f);
            }
        }
    }

    bool CheckOverLine(Vector3 cPos) {
        return CheckXLine(cPos.x) || CheckYLine(cPos.y) || CheckZLine(cPos.z);
    }

    bool CheckXLine(float x) {
        return x > PinWallMAX.x || x < PinWallMIN.x;
    }
    bool CheckYLine(float y) {
        return y > PinWallMAX.y || y < PinWallMIN.y;
    }
    bool CheckZLine(float z) {
        return z > PinWallMAX.z || z < PinWallMIN.z;
    }

    public int GetNowFramePinNum() { return nFramePinNum; }

    void SetNowFramePinNum() {
        nFramePinNum = nAllPinNum - nNumStandPin;
    }
}
